import { isUndefined } from '@guardian/libs';

const typographyObjChanges = ["body", "headline", "textSans", "titlepiece"];
const removedImports = {
  "'@guardian/source-foundations/utils'": ["InteractionModeEngine"],
  "'@guardian/src-foundations/size/global'": ["remSize", "remIconSize"],
  "'@guardian/src-foundations/themes'": ["defaultTheme", "brand", "brandAlt"],
  "'@guardian/src-helpers'": [
    "storybookBackgrounds",
    "ThemeName",
    "svgBackgroundImage"
  ]
};
const newThemeNames = {
  accordionDefault: "accordionThemeDefault",
  buttonReaderRevenue: "buttonThemeReaderRevenue",
  buttonReaderRevenueBrand: "buttonThemeReaderRevenueBrand",
  buttonReaderRevenueBrandAlt: "buttonThemeReaderRevenueBrandAlt",
  buttonBrand: "buttonThemeBrand",
  buttonBrandAlt: "buttonThemeBrandAlt",
  buttonDefault: "buttonThemeDefault",
  checkboxBrand: "checkboxThemeBrand",
  checkboxDefault: "checkboxThemeDefault",
  choiceCardDefault: "choiceCardThemeDefault",
  footerBrand: "footerThemeBrand",
  labelDefault: "labelThemeDefault",
  labelBrand: "labelThemeBrand",
  linkBrand: "linkThemeBrand",
  linkBrandAlt: "linkThemeBrandAlt",
  linkDefault: "linkThemeDefault",
  radioBrand: "radioThemeBrand",
  radioDefault: "radioThemeDefault",
  selectDefault: "selectThemeDefault",
  textInputDefault: "textInputThemeDefault",
  userFeedbackBrand: "userFeedbackThemeBrand",
  userFeedbackDefault: "userFeedbackThemeDefault"
};
const capitalise = (str) => str.charAt(0).toUpperCase() + str.slice(1);
const getNewPackage = (oldPackage) => {
  if (oldPackage === "'@guardian/src-foundations/themes'") {
    return "'@guardian/source-react-components'";
  } else if (oldPackage.startsWith("'@guardian/src-foundations")) {
    return "'@guardian/source-foundations'";
  } else {
    return "'@guardian/source-react-components'";
  }
};
const getRemovedExports = (node) => {
  switch (node.type) {
    case "ImportDeclaration":
      return node.specifiers.filter((i) => {
        const source = node.source.raw;
        if (!source || !Object.keys(removedImports).includes(source)) {
          return false;
        }
        const removedImportsForSource = removedImports[source];
        if (removedImportsForSource) {
          return i.type === "ImportSpecifier" && removedImportsForSource.includes(i.imported.name);
        }
        return false;
      });
    case "ExportNamedDeclaration":
      return node.specifiers.filter((i) => {
        const source = node.source?.raw;
        if (!source || !Object.keys(removedImports).includes(source)) {
          return false;
        }
        const removedImportsForSource = removedImports[source];
        if (removedImportsForSource) {
          return removedImportsForSource.includes(i.exported.name);
        }
        return false;
      });
    case "ExportAllDeclaration":
      return [];
  }
};
const getImportName = (i) => {
  const imported = i.type === "ImportSpecifier" ? i.imported.name : i.exported.name;
  const local = i.local.name;
  return imported === local ? imported : `${imported} as ${local}`;
};
const getSpecifierName = (i) => {
  return i.type === "ImportSpecifier" ? i.imported.name : i.exported.name;
};
const getRenameImportFixers = (node, removedExports, fixer, nodeSource) => {
  const fixers = [];
  if (!node.source?.raw || node.type === "ExportAllDeclaration") {
    return fixers;
  }
  if (removedExports.length) {
    for (const i of removedExports) {
      if (!i.range) {
        break;
      }
      const end = i.range[1];
      const comma = nodeSource.slice(end, end + 1);
      fixers.push(fixer.removeRange([i.range[0], comma ? end + 1 : end]));
    }
    const importsArray = [];
    for (const rExport of removedExports) {
      const newName = getImportName(rExport);
      importsArray.push(newName);
    }
    fixers.push(
      fixer.insertTextBeforeRange(
        node.range ?? [0, 0],
        `import { ${importsArray.join(", ")} } from ${node.source.raw};
`
      )
    );
  }
  return fixers;
};
const getMessage = (newPackage, removedExports, node) => {
  const importOrExport = node.type.startsWith("Export") ? "export" : "import";
  const newPackageMessage = `@guardian/src-* packages are deprecated. ${capitalise(
    importOrExport
  )} from ${newPackage} instead.`;
  if (node.type === "ExportAllDeclaration") {
    return newPackageMessage;
  }
  const renamedExports = [];
  if (node.source?.raw === "'@guardian/src-foundations/typography/obj'") {
    for (const i of node.specifiers) {
      if (i.type === "ImportNamespaceSpecifier" || i.type === "ImportDefaultSpecifier") {
        continue;
      }
      const name = getSpecifierName(i);
      if (typographyObjChanges.includes(name)) {
        renamedExports.push([name, `${name}ObjectStyles`]);
      }
    }
  }
  for (const i of node.specifiers) {
    if (i.type === "ImportNamespaceSpecifier" || i.type === "ImportDefaultSpecifier") {
      continue;
    }
    const name = getSpecifierName(i);
    const newThemeName = newThemeNames[name];
    if (!isUndefined(newThemeName)) {
      renamedExports.push([name, `${newThemeName}`]);
    }
  }
  let renamedExportsMessage = "";
  if (renamedExports.length) {
    renamedExportsMessage = `The following export(s) have been renamed [from -> to]: ${renamedExports.map(([oldName, newName]) => `${oldName} -> ${newName}`).join(", ")}`;
  }
  if (!removedExports.length) {
    return renamedExportsMessage ? `${newPackageMessage}
${renamedExportsMessage}` : newPackageMessage;
  }
  const totalImports = node.specifiers.length;
  const removedExportsArray = [];
  for (const rExport of removedExports) {
    removedExportsArray.push(
      rExport.type === "ImportSpecifier" ? rExport.imported.name : rExport.exported.name
    );
  }
  const removedExportsMessage = `The following export(s) have been removed: ${removedExportsArray.join(
    ", "
  )}.`;
  if (totalImports === removedExports.length) {
    return removedExportsMessage;
  } else {
    return [newPackageMessage, removedExportsMessage, renamedExportsMessage].filter((s) => !!s).join("\n");
  }
};
const relevantImportSource = (importSource, pkg) => {
  if (importSource.startsWith("'@guardian/src-foundations")) {
    return true;
  }
  return pkg === "all" ;
};
const createReport = (context, node, pkg) => {
  const importSource = node.source?.raw;
  if (!importSource || !relevantImportSource(importSource, pkg)) {
    return;
  }
  const newPackage = getNewPackage(importSource);
  if (node.type === "ExportAllDeclaration") {
    return context.report({
      node,
      message: getMessage(newPackage, [], node)
    });
  }
  const nonNamedImports = node.type === "ImportDeclaration" && !node.specifiers.every((s) => s.type === "ImportSpecifier");
  if (nonNamedImports) {
    return context.report({
      node,
      message: getMessage(newPackage, [], node)
    });
  }
  const removedExports = getRemovedExports(node);
  const nodeSource = context.getSourceCode().getText(node);
  return context.report({
    node,
    message: getMessage(newPackage, removedExports, node),
    fix: (fixer) => {
      return node.specifiers.length === removedExports.length ? null : [
        ...getRenameImportFixers(node, removedExports, fixer, nodeSource),
        fixer.replaceTextRange(node.source?.range ?? [0, 0], newPackage)
      ];
    }
  });
};
const validFoundationsImportPath = {
  meta: {
    type: "problem",
    docs: {
      description: "Get Source imports from v4 packages",
      category: "Deprecated",
      url: "https://github.com/guardian/csnx/tree/main/libs/@guardian/eslint-plugin-source-foundations/README.md"
    },
    fixable: "code",
    schema: []
  },
  create: (context) => {
    return {
      ImportDeclaration(node) {
        return createReport(context, node, "foundations");
      },
      ExportNamedDeclaration(node) {
        return createReport(context, node, "foundations");
      },
      ExportAllDeclaration(node) {
        return createReport(context, node, "foundations");
      }
    };
  }
};

export { validFoundationsImportPath };
